/*global define*/
/*jslint vars:true*/
define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/externalcontent/ui/externalvideo-test.html',
    'jquery',

    'de_epages/externalcontent/ui/externalvideo',
    '$ready!'
], function (QUnit, fixture, $) {
    'use strict';

    QUnit.module('de_epages/externalcontent/ui/externalvideo', {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.test('Test externalvideo methods', 2, function () {
        $.each(['_create', 'destroy'], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.externalcontentUiExternalvideo.prototype[fn]),
                true,
                fn + ' is a method of externalvideo'
            );
        });
    });

    QUnit.test('Test externalvideo', 26, function () {
        $("#externalvideoplugin1").externalcontentUiExternalvideo({
            provider: "youtube",
            id: "UGhd7eZfrDA",
            width: 320,
            height: 180
        });

        var youtubePlayer = $("#externalvideoplugin1").find(":first-child");

        QUnit.equal(
            youtubePlayer.attr("src"),
            "//www.youtube.com/embed/UGhd7eZfrDA",
            "the plugin is appended to the correct div and has the correct source url"
        );

        QUnit.equal(
            youtubePlayer.prop("tagName"),
            "IFRAME",
            "the plugin is appended with the correct tag"
        );

        QUnit.equal(
            youtubePlayer.attr("width"),
            "320",
            "the appended iframe has the correct width"
        );

        QUnit.equal(
            youtubePlayer.attr("height"),
            "180",
            "the appended iframe has the correct height"
        );

        $("#externalvideoplugin2").externalcontentUiExternalvideo({
            provider: "wideofr",
            id: "bfd446a822as"
        });

        var wideoPlayer = $("#externalvideoplugin2").find(":first-child");

        QUnit.equal(
            wideoPlayer.attr("id"),
            "bfd446a822as",
            "the plugin is appended to the correct div and has the correct source id"
        );

        QUnit.equal(
            wideoPlayer.prop("tagName"),
            "OBJECT",
            "the plugin is appended with the correct tag"
        );

        QUnit.equal(
            wideoPlayer.attr("width"),
            "560",
            "the appended iframe has the correct width"
        );

        QUnit.equal(
            wideoPlayer.attr("height"),
            "315",
            "the appended iframe has the correct height"
        );

        $("#externalvideoplugin3").externalcontentUiExternalvideo({
            provider: "myvideo",
            id: "8092203"
        });

        var myvideoPlayer = $("#externalvideoplugin3").find(":first-child");

        QUnit.equal(
            myvideoPlayer.attr("src"),
            "//www.myvideo.de/embed/8092203",
            "the plugin is appended to the correct div and has the correct source id"
        );

        QUnit.equal(
            myvideoPlayer.prop("tagName"),
            "IFRAME",
            "the plugin is appended with the correct tag"
        );

        QUnit.equal(
            myvideoPlayer.attr("width"),
            "560",
            "the appended iframe has the correct width"
        );

        QUnit.equal(
            myvideoPlayer.attr("height"),
            "315",
            "the appended iframe has the correct height"
        );

        $("#externalvideoplugin4").externalcontentUiExternalvideo({
            provider: "vimeo",
            id: "45824348",
            width: 320,
            height: 180
        });

        var vimeoPlayer = $("#externalvideoplugin4").find(":first-child");

        QUnit.equal(
            vimeoPlayer.attr("src"),
            "//player.vimeo.com/video/45824348",
            "the plugin is appended to the correct div and has the correct source id"
        );

        QUnit.equal(
            vimeoPlayer.prop("tagName"),
            "IFRAME",
            "the plugin is appended with the correct tag"
        );

        QUnit.equal(
            vimeoPlayer.attr("width"),
            "320",
            "the appended iframe has the correct width"
        );

        QUnit.equal(
            vimeoPlayer.attr("height"),
            "180",
            "the appended iframe has the correct height"
        );

        $("#externalvideoplugin5").externalcontentUiExternalvideo({
            provider: "dailymotion",
            id: "xqkho0"
        });

        var dailyPlayer = $("#externalvideoplugin5").find(":first-child");

        QUnit.equal(
            dailyPlayer.attr("src"),
            "//www.dailymotion.com/embed/video/xqkho0",
            "the plugin is appended to the correct div and has the correct source id"
        );

        QUnit.equal(
            dailyPlayer.prop("tagName"),
            "IFRAME",
            "the plugin is appended with the correct tag"
        );

        QUnit.equal(
            dailyPlayer.attr("width"),
            "560",
            "the appended iframe has the correct width"
        );

        QUnit.equal(
            dailyPlayer.attr("height"),
            "315",
            "the appended iframe has the correct height"
        );

        $("#externalvideoplugin6").externalcontentUiExternalvideo({
            provider: "metacafe",
            id: "yt-2B1QxOmbMZ0/alf_tries_to_eat_the_cat",
            width: 440,
            height: 272
        });

        var metacafePlayer = $("#externalvideoplugin6").find(":first-child");

        QUnit.equal(
            metacafePlayer.attr("src"),
            "http://www.metacafe.com/fplayer/yt-2B1QxOmbMZ0/alf_tries_to_eat_the_cat.swf",
            "the plugin is appended to the correct div and has the correct source url"
        );

        QUnit.equal(
            metacafePlayer.prop("tagName"),
            "EMBED",
            "the plugin is appended with the correct tag"
        );

        QUnit.equal(
            metacafePlayer.attr("width"),
            "440",
            "the appended iframe has the correct width"
        );

        QUnit.equal(
            metacafePlayer.attr("height"),
            "272",
            "the appended iframe has the correct height"
        );

        $("div[id^=externalvideoplugin]").each(function () {
            $(this).externalcontentUiExternalvideo("destroy");
        });

        var hasClassVideoGadget = true;

        $("div[id^=externalvideoplugin]").each(function () {
            var value = $(this).hasClass("VideoGadget");
            if (hasClassVideoGadget !== value) {
                hasClassVideoGadget = value;
            }
        });

        var ChildNodes = 1;
        $("div[id^=externalvideoplugin]").each(function () {
            var value = $(this).children().length;
            if (ChildNodes !== value) {
                ChildNodes = value;
            }
        });

        QUnit.equal(
            hasClassVideoGadget,
            false,
            "the destroy method has successfully deleted the class SocialWeb from every <<Div>> "
        );

        QUnit.equal(
            ChildNodes,
            0,
            "the destroy method has successfully disappended all child elements, length of child elements from each <<Div>> = 0"
        );
    });
});
