/*globals require, window*/

require([
    'jquery',
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/shop/ui/languageswitch-test',

    'de_epages/shop/ui/languageswitch'
], function ($, QUnit, fixture) {

    QUnit.module('de_epages/shop/ui/languageswitch', {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest('check presence/existence of methods', 7, function () {
        // check prototype methods
        $.each([
            '_init',
            '_createDialog',
            '_prepareEnvironment',
            '_getPosition',
            'open',
            'close',
            'destroy'
        ], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.shopUiLanguageswitch.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );

        });

        // run tests
        QUnit.start();
    });

    QUnit.asyncTest('test widget functionalities', function () {
        var $languageSwitch = $('.ep-top-bar-language-select-languageswitch'),
            languages = [{
                "code": "ger",
                "value": "1",
                "translation": "Deutsch",
                "selected": true
            }, {
                "code": "eng",
                "value": "2",
                "translation": "Englisch",
                "selected": false
            }],

            activeItemClass = 'ep-languageswitch-list-item-active',

            languageSwitchWidget,
            $languagesList,
            languagesListItems,
            $selectedLanguage,
            selectedLanguageData,
            $notSelectedLanguage,
            notSelectedLanguageData;


        $languageSwitch.shopUiLanguageswitch({
            languages: languages,
            // fake location test by adding the variable language id via url hash
            location: '#AltLanguageID=',
            ofElement: $languageSwitch.closest('.ep-top-bar'),
            position: {
                my: "left top",
                at: "left bottom+3"
            },
            translations: {
                tInfoHeader: 'headline',
                tInfoDescription: 'description'
            }
        });

        languageSwitchWidget = $languageSwitch.data('uiShopUiLanguageswitch');

        // start testing
        QUnit.equal(
            languageSwitchWidget instanceof $.Widget,
            true,
            "shopUiLanguageswitch widget was created and is an instance of jQuery.Widget"
        );

        // open language switch dialog
        $languageSwitch.trigger('click');
        setTimeout(function() {
            $languagesList = languageSwitchWidget.dialog.find('ul');
            languagesListItems = languageSwitchWidget.dialog.find('li');

            $selectedLanguage = languagesListItems.first();
            selectedLanguageData = $selectedLanguage.data('language');
            $notSelectedLanguage = languagesListItems.last();
            notSelectedLanguageData = $notSelectedLanguage.data('language');

            QUnit.equal(
                languageSwitchWidget.dialog.is(':visible'),
                true,
                "Dialog with language list is open"
            );
            QUnit.equal(
                languageSwitchWidget.dialog.uiDialog().closest('.ui-dialog').hasClass('ui-dialog-headless'),
                true,
                "Dialog had additional class 'ui-dialog-headless'."
            );
            // check number of languages in list
            QUnit.equal(
                languagesListItems.length,
                languages.length,
                "Language list contains " + languages.length + " languages"
            );
            // check language data
            QUnit.equal(
                JSON.stringify(selectedLanguageData),
                JSON.stringify(languages[0]),
                "Data of first language from list item is correct: " + JSON.stringify(languages[0])
            );
            QUnit.equal(
                JSON.stringify(notSelectedLanguageData),
                JSON.stringify(languages[1]),
                "Data of second language from list item is correct: " + JSON.stringify(languages[1])
            );
            // check selected language
            QUnit.equal(
                JSON.stringify(selectedLanguageData.selected) === 'true',
                true,
                "First language is selected."
            );
            // check icon element
            QUnit.equal(
                $selectedLanguage.find('i.fa').length,
                1,
                "Selected language contains an icon element."
            );
            QUnit.equal(
                $selectedLanguage.hasClass(activeItemClass),
                true,
                "Selected language element has class '" + activeItemClass + "'."
            );
            QUnit.equal(
                $notSelectedLanguage.find('i.fa').length,
                0,
                "Not selected language doesn't contain an icon element."
            );

            $languagesList.trigger('mouseenter');
            QUnit.equal(
                $selectedLanguage.hasClass(activeItemClass),
                false,
                "After entering the language list with the cursor, the selected language element hasn't the class '" + activeItemClass + "'."
            );
            $languagesList.trigger('mouseleave');
            QUnit.equal(
                $selectedLanguage.hasClass(activeItemClass),
                true,
                "After leaving the language list with the cursor, the selected language element has the class '" + activeItemClass + "' again."
            );

            // close dialog via public method
            $languageSwitch.shopUiLanguageswitch('close');

            setTimeout(function() {
                QUnit.equal(
                    languageSwitchWidget.dialog.is(':visible'),
                    false,
                    "After clicking outside the dialog with language list, the dialog is closed."
                );

                // open dialog via public method
                $languageSwitch.shopUiLanguageswitch('open');

                setTimeout(function() {
                    QUnit.equal(
                        languageSwitchWidget.dialog.is(':visible'),
                        true,
                        "Dialog is open after using the public widget method 'open'."
                    );

                    // trigger click on not selected language
                    $notSelectedLanguage.trigger('click');
                    QUnit.equal(
                        (new RegExp("AltLanguageID="+notSelectedLanguageData.value)).test(window.location.href),
                        true,
                        "URL contains correct language id."
                    );

                    // close dialog via public method
                    $languageSwitch.shopUiLanguageswitch('close');

                    $languageSwitch.shopUiLanguageswitch('destroy');

                    setTimeout(function() {
                        QUnit.equal(
                            $languageSwitch.data('uiShopUiLanguageswitch') === undefined,
                            true,
                            "Language switch wiget was destroyed."
                        );
                        QUnit.equal(
                            $('.ui-dialog-headless').length,
                            0,
                            "Dialog was destroyed."
                        );

                        // run tests
                        QUnit.start();
                    }, 100);
                }, 300);
            }, 100);
        }, 300);
    });
});