define([
    'testsuite/qunit',
    'testsuite/sinon',
    'jquery',
    'ep/ui/slide-panel',
    "$dict!ep/dict",

    '$ready!'
], function (QUnit, sinon, $, ep, epDict) {

    // translations object
    var translations = {
            GoToCartLink: epDict.translate('GoToCartLink'),
            ProductAddedToCart: epDict.translate('ProductAddedToCart')
        };

    QUnit.asyncTest("widget test presence of methods", function () {
        // test presence of methods#
        $.each([
            '_create',
            '_setType',
            '_setVisibility',
            '_setContent',
            '_setOption',
            '_createPanel',
            'show',
            'hide',
            'destroy'
        ], function (i, fn) {
                QUnit.equal(
                $.isFunction($.ui.uiSlidePanel.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest("widget ui tests", function () {
        // initialize widget
        var $slidePanel = $('<div />').uiSlidePanel({
                duration: 300
            }),

            content = {
                notification: 'Another notification message.',
                link: {
                    text: 'Another link text',
                    href: '/another'
                }
            },

            hideSpy;

        setTimeout(function() {
            // test presence of widget
            QUnit.ok(
                $slidePanel.data('uiUiSlidePanel') !== undefined,
                'Widget exists.'
            );

            QUnit.ok(
                $('.add-to-cart-notice').length === 1,
                'Slide panel was created.'
            );

            // register spy to investigate hide method
            hideSpy = sinon.spy($slidePanel.data('uiUiSlidePanel'), '_setVisibility');

            // check default options
            QUnit.ok(
                $slidePanel.hasClass('add-to-cart-notice-hidden'),
                'Slide panel is hidden.'
            );
            QUnit.ok(
                $slidePanel.hasClass('add-to-cart-notice-success'),
                'Slide panel has success class by default.'
            );
            QUnit.equal(
                $('.add-to-cart-notice-message span span').text(),
                translations.ProductAddedToCart,
                'Found correct default notification message.'
            );
            QUnit.equal(
                $('.add-to-cart-notice-message span a').text(),
                translations.GoToCartLink,
                'Found correct default link text.'
            );
            QUnit.equal(
                $('.add-to-cart-notice-message span a').attr('href'),
                '/cart',
                'Found correct default link text.'
            );

            // show slide panel
            $slidePanel.uiSlidePanel('show');
            QUnit.ok(
                hideSpy.callCount === 1,
                'Set visibility method was called the 1st time.'
            );

            QUnit.ok(
                $slidePanel.hasClass('add-to-cart-notice-visible'),
                'Slide panel is visible.'
            );

            // wait for closing the panel automatically
            setTimeout(function() {
                QUnit.ok(
                    $slidePanel.hasClass('add-to-cart-notice-hidden'),
                    'Slide panel is hidden again automatically after a duration of 300ms.'
                );
                QUnit.ok(
                    hideSpy.callCount === 2,
                    'Set visibility method was called the 2nd time.'
                );

                // set duration to a longer period and check public hide method
                $slidePanel.uiSlidePanel('option', 'duration', 5000);
                // show slide panel
                $slidePanel.uiSlidePanel('show');
                QUnit.ok(
                    hideSpy.callCount === 3,
                    'Set visibility method was called the 3rd time.'
                );
                QUnit.ok(
                    $slidePanel.hasClass('add-to-cart-notice-visible'),
                    'Slide panel is visible.'
                );
                // hide slide panel manually
                $slidePanel.uiSlidePanel('hide');
                QUnit.ok(
                    hideSpy.callCount === 4,
                    'Set visibility method was called the 4th time.'
                );
                QUnit.ok(
                    $slidePanel.hasClass('add-to-cart-notice-hidden'),
                    'Slide panel was closed by using the hide method.'
                );

                // test closing by using the close button
                // show slide panel
                $slidePanel.uiSlidePanel('show');
                QUnit.ok(
                    hideSpy.callCount === 5,
                    'Set visibility method was called the 5th time.'
                );
                QUnit.ok(
                    $slidePanel.hasClass('add-to-cart-notice-visible'),
                    'Slide panel is visible.'
                );
                // hide slide panel manually
                $slidePanel.find('.add-to-cart-notice-close').trigger('click');
                QUnit.ok(
                    $slidePanel.hasClass('add-to-cart-notice-hidden'),
                    'Slide panel was closed by using the close button.'
                );
                QUnit.ok(
                    hideSpy.callCount === 6,
                    'Set visibility method was called the 6th time.'
                );

                // test option settings
                // set type option to existing type 'failure'
                $slidePanel.uiSlidePanel('option', 'type', 'failure');
                QUnit.ok(
                    $slidePanel.hasClass('add-to-cart-notice-failure'),
                    'Slide panel has type *failure* after setting the corresponding option.'
                );
                // set type option to an unknown type 'unknown'
                $slidePanel.uiSlidePanel('option', 'type', 'unknown');
                QUnit.ok(
                    $slidePanel.hasClass('add-to-cart-notice-unknown'),
                    'Slide panel has type *unknown* after setting the corresponding option.'
                );

                // set content
                // set notification message
                $slidePanel.uiSlidePanel('option', 'notification', content.notification);
                // set link options
                $slidePanel.uiSlidePanel('option', 'link', content.link);
                QUnit.equal(
                    $('.add-to-cart-notice-message span span').text(),
                    content.notification,
                    'Found new notification message.'
                );
                QUnit.equal(
                    $('.add-to-cart-notice-message span a').text(),
                    content.link.text,
                    'Found new link text.'
                );
                QUnit.equal(
                    $('.add-to-cart-notice-message span a').attr('href'),
                    content.link.href,
                    'Found new href.'
                );
                QUnit.ok(
                    $('.add-to-cart-notice-message span a').is(':visible'),
                    'Link is visible'
                );

                // set link options => link becomes invisible
                $slidePanel.uiSlidePanel('option', 'link', false);
                QUnit.ok(
                    $('.add-to-cart-notice-message span a').is(':visible') === false,
                    'Link is hidden'
                );

                // Destroy widget
                $slidePanel.uiSlidePanel('destroy');

                setTimeout(function() {
                    QUnit.ok(
                        $slidePanel.data('uiUiSlidePanel') === undefined,
                        'Widget destroyed.'
                    );

                    QUnit.ok(
                        $('.add-to-cart-notice').length === 0,
                        'Slide panel node was removed from DOM.'
                    );

                    // start async tests
                    QUnit.start();
                }, 300);
            }, 320);
        }, 500);
    });
});