/*global define, window, ep, reset*/
/*jslint nomen:true*/
define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/externalcontent/ui/twitter-test.html',
    'jquery',
    'ep',

    'de_epages/externalcontent/ui/twitter',
    '$ready!'
], function (QUnit, fixture, $, ep) {
    'use strict';

    QUnit.module('de_epages/externalcontent/ui/twitter', {
        setup: function () {
            fixture.init();

            ep.config = {
                javascriptRoot: '.',
                debugLevel: 2,
                canonicalUrl: 'www.canonical.de',
                webUrl: 'www.webUrl.de'
            };

            // Twitter *TWTR* API for testing.
            window.TWTR = {
                Widget: function (params) {
                    var self = this;
                    this.render = function () {
                        window.isrendered = true;
                        return self;
                    };
                    this.start = function () {
                        window.isstarted = true;
                        return self;
                    };
                    this.setUser = function (username) {
                        window.username = username;
                        return self;
                    };
                    this.destroy = function () {
                        window.isdestroyed = true;
                    };
                    window.params = params;
                }
            };

            // Twitter *twttr* API for testing.
            window.twttr = {
                widgets: {
                    load: function () {
                        window.isloaded = true;
                    }
                }
            };

            // Reset test values.
            window.reset = function () {
                window.wasrendered = false;
                window.wasstarted = false;
                window.username = "";
                window.isdestroyed = false;
                window.isloaded = false;
                window.params = {};
            };
        },

        teardown: fixture.destroy
    });

    QUnit.asyncTest('Test twitter methods', 3, function () {
        $.each(['_create', '_twitterButton', 'destroy'], function (i, fn) {
            QUnit.equal(
                $.isFunction($.ui.externalcontentUiTwitter.prototype[fn]),
                true,
                fn + ' is a method of twitter'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest('Test twitter', 7, function () {
        var responseString = '["?ViewAction=DereferredRedirect&URL=https%3A//twitter.com/share&nonce=YWYwN2E4NGUzNDc1OTNhNGIxNTU2YjYwZTc3ZjEwMDYyOTAxOTI0Yjg4ZWM5MzJjMDBlZGUzM2MxYjA5MjgxN18xNDg2NDc5MjI2"]',
            response = JSON.parse(responseString),

            userName = "Boobs",
            responseStringBoobs = '["?ViewAction=DereferredRedirect&URL=https%3A//twitter.com/https%3A//twitter.com/Boobs&nonce=MGFjMzM1ZmY5NzEzZmQ3YzIxODZlZDBkYjEzNGQwOTFkM2RlNDUxZGE2MWNkMDBhM2RjYTQwN2I3NWVhY2E4ZF8xNDg2NDgwNDky"]',
            responseBoobs = JSON.parse(responseStringBoobs);

        /* START - fake ajax request */
        ep.ajax = function (o) {
            var result = $.Deferred(),
                resolveSend;

            if (o.data && o.data.ViewAction) {

                if (o.data.ViewAction === "JSONDerefer") {
                    if (o.data.DereferURLs === "https://twitter.com/share") {
                        result.resolve(response);
                    } else if (o.data.DereferURLs === ("https://twitter.com/" + userName)) {
                        result.resolve(responseBoobs);
                    }
                }

            } else if (o && o.dataType === "script") {
                result.resolve(true);
            } else {
                result.reject();
            }

            return result;
        };
        /* END - fake ajax request */

        // Create *twitter* (share) gadget. (twttr API, see above)
        $("#twitterbutton1").externalcontentUiTwitter({
            url: "www.testurl1.de",
            type: "share"
        });

        QUnit.equal(
            window.isloaded,
            true,
            "the Twitter function loading the share button in the target div has been called"
        );

        QUnit.equal(
            $("#twitterbutton1").find("a").attr("class"),
            "twitter-share-button",
            "the appended anchor has the class twitter-share-button"
        );

        QUnit.equal(
            $("#twitterbutton1").find("a").attr("data-url"),
            "www.testurl1.de",
            "the passed url-parameter has been added into the data-url attribute"
        );

        reset();

        // Create *twitter* (follow) gadget. (twttr API, see above)
        $("#twitterbutton2").externalcontentUiTwitter({
            username: userName,
            type: "follow"
        });

        QUnit.equal(
            window.isloaded,
            true,
            "the Twitter function loading the follow button in the target div has been called"
        );

        QUnit.equal(
            $("#twitterbutton2").find("a").attr("class"),
            "twitter-follow-button",
            "the appended anchor has the class twitter-follow-button"
        );

        QUnit.equal(
            (new RegExp("twitter.com/" + userName)).test($("#twitterbutton2").find("a").attr("href")),
            true,
            "the passed username-parameter has been added into the visible anchor font"
        );

        reset();

        // Destroy *twitter* (follow) gadget.
        $("#twitterbutton2").externalcontentUiTwitter("destroy");

        QUnit.equal(
            $("#twitterbutton2").children().length,
            0,
            "The container div was removed from *self.element*"
        );

        reset();

        QUnit.start();
    });
});